# Copyright iris-grib contributors
#
# This file is part of iris-grib and is released under the LGPL license.
# See COPYING and COPYING.LESSER in the root of the repository for full
# licensing details.
"""
Integration tests for format interoperability.

Checks that specific features of cubes loaded from other formats are correctly
represented by grib saving.

"""
